@@ -41,13 +41,16 @@
 #include "prerror.h"
 #include "prprf.h"
 #include "plgetopt.h"
+#include "prenv.h"
 
 #include "secutil.h"
 #include "secpkcs7.h"
 #include "secrng.h"
-#include <sys/stat.h>
 #include <stdarg.h>
+#if !defined(_WIN32_WCE)
+#include <sys/stat.h>
 #include <errno.h>
+#endif
 
 #ifdef XP_UNIX
 #include <unistd.h>
@@ -69,9 +72,13 @@ static char consoleName[] =  {
 #else
     "/dev/tty"
 #endif
+#else
+#ifdef XP_OS2
+    "\\DEV\\CON"
 #else
     "CON:"
 #endif
+#endif
 };
 
 char *
@@ -110,7 +117,11 @@ SECU_PrintSystemError(char *progName, char *msg, ...)
     va_start(args, msg);
     fprintf(stderr, "%s: ", progName);
     vfprintf(stderr, msg, args);
+#if defined(_WIN32_WCE)
+    fprintf(stderr, ": %d\n", PR_GetOSError());
+#else
     fprintf(stderr, ": %s\n", strerror(errno));
+#endif
     va_end(args);
 }
 
@@ -258,7 +269,8 @@ secu_InitSlotPassword(PK11SlotInfo *slot, PRBool retry, void *arg)
 
     if (pwdata->source == PW_FROMFILE) {
 	return SECU_FilePasswd(slot, retry, pwdata->data);
-    } else if (pwdata->source == PW_PLAINTEXT) {
+    } 
+    if (pwdata->source == PW_PLAINTEXT) {
 	return PL_strdup(pwdata->data);
     }
     
@@ -275,12 +287,10 @@ secu_InitSlotPassword(PK11SlotInfo *slot, PRBool retry, void *arg)
     }
 
     /* we have no password, so initialize database with one */
-    PR_fprintf(PR_STDERR, "In order to finish creating your database, you\n");
-    PR_fprintf(PR_STDERR, "must enter a password which will be used to\n");
-    PR_fprintf(PR_STDERR, "encrypt this key and any future keys.\n\n");
-    PR_fprintf(PR_STDERR, "The password must be at least 8 characters long,\n");
-    PR_fprintf(PR_STDERR, "and must contain at least one non-alphabetic ");
-    PR_fprintf(PR_STDERR, "character.\n\n");
+    PR_fprintf(PR_STDERR, 
+        "Enter a password which will be used to encrypt your keys.\n"
+     	"The password should be at least 8 characters long,\n"
+     	"and should contain at least one non-alphabetic character.\n\n");
 
     output = fopen(consoleName, "w");
     if (output == NULL) {
@@ -290,15 +300,16 @@ secu_InitSlotPassword(PK11SlotInfo *slot, PRBool retry, void *arg)
 
 
     for (;;) {
-	if (!p0) {
-	    p0 = SEC_GetPassword(input, output, "Enter new password: ",
-			         SEC_BlindCheckPassword);
-	}
-	if (pwdata->source == PW_NONE) {
-	    p1 = SEC_GetPassword(input, output, "Re-enter password: ",
-				 SEC_BlindCheckPassword);
-	}
-	if (pwdata->source != PW_NONE || (PORT_Strcmp(p0, p1) == 0)) {
+	if (p0) 
+	    PORT_Free(p0);
+	p0 = SEC_GetPassword(input, output, "Enter new password: ",
+			     SEC_BlindCheckPassword);
+
+	if (p1)
+	    PORT_Free(p1);
+	p1 = SEC_GetPassword(input, output, "Re-enter password: ",
+			     SEC_BlindCheckPassword);
+	if (p0 && p1 && !PORT_Strcmp(p0, p1)) {
 	    break;
 	}
 	PR_fprintf(PR_STDERR, "Passwords do not match. Try again.\n");
@@ -388,7 +399,7 @@ SECU_DefaultSSLDir(void)
     char *dir;
     static char sslDir[1000];
 
-    dir = getenv("SSL_DIR");
+    dir = PR_GetEnv("SSL_DIR");
     if (!dir)
 	return NULL;
 
@@ -424,7 +435,7 @@ SECU_ConfigDirectory(const char* base)
     
 
     if (base == NULL || *base == 0) {
-	home = getenv("HOME");
+	home = PR_GetEnv("HOME");
 	if (!home) home = "";
 
 	if (*home && home[strlen(home) - 1] == '/')
@@ -673,7 +684,7 @@ static void secu_Newline(FILE *out)
 }
 
 void
-SECU_PrintAsHex(FILE *out, SECItem *data, char *m, int level)
+SECU_PrintAsHex(FILE *out, SECItem *data, const char *m, int level)
 {
     unsigned i;
     int column;
@@ -693,13 +704,13 @@ SECU_PrintAsHex(FILE *out, SECItem *data, char *m, int level)
 	}
 	if (i != data->len - 1) {
 	    fprintf(out, "%02x:", data->data[i]);
-	    column += 4;
+	    column += 3;
 	} else {
 	    fprintf(out, "%02x", data->data[i]);
-	    column += 3;
+	    column += 2;
 	    break;
 	}
-	if (column > 76) {
+	if (column > 76 || (i % 16 == 15)) {
 	    secu_Newline(out);
 	    SECU_Indent(out, level); column = level*INDENT_MULT;
 	}
@@ -787,7 +798,14 @@ SECU_PrintInteger(FILE *out, SECItem *i, char *m, int level)
 {
     int iv;
 
-    if (i->len > 4) {
+    if (!i || !i->len || !i->data) {
+	SECU_Indent(out, level); 
+	if (m) {
+	    fprintf(out, "%s: (null)\n", m);
+	} else {
+	    fprintf(out, "(null)\n");
+	}
+    } else if (i->len > 4) {
 	SECU_PrintAsHex(out, i, m, level);
     } else {
 	iv = DER_GetInteger(i);
@@ -962,6 +980,7 @@ SECU_PrintSet(FILE *out, SECItem *t, char *m, int level)
     }
     /* { */SECU_Indent(out, level); fprintf(out, "}\n");
 }
+
 static void
 secu_PrintContextSpecific(FILE *out, SECItem *i, char *m, int level)
 {
@@ -974,7 +993,7 @@ secu_PrintContextSpecific(FILE *out, SECItem *i, char *m, int level)
     	fprintf(out, "%s: ", m);
     }
 
-    fprintf(out,"Option %d\n", type);
+    fprintf(out,"[%d]\n", type);
     start = 2;
     if (i->data[1] & 0x80) {
 	start = (i->data[1] & 0x7f) +1;
@@ -984,6 +1003,42 @@ secu_PrintContextSpecific(FILE *out, SECItem *i, char *m, int level)
     SECU_PrintAsHex(out, &tmp, m, level+1);
 }
 
+static void
+secu_PrintOctetString(FILE *out, SECItem *i, char *m, int level)
+{
+    SECItem tmp;
+    int start;
+
+    start = 2;
+    if (i->data[1] & 0x80) {
+	start = (i->data[1] & 0x7f) +1;
+    }
+    tmp.data = &i->data[start];
+    tmp.len = i->len - start;
+    SECU_PrintAsHex(out, &tmp, m, level);
+}
+
+static void
+secu_PrintBitString(FILE *out, SECItem *i, char *m, int level)
+{
+    SECItem tmp;
+    int start;
+    int unused_bits;
+
+    start = 2;
+    if (i->data[1] & 0x80) {
+	start = (i->data[1] & 0x7f) + 1;
+    }
+    unused_bits = i->data[start++];
+    tmp.data = &i->data[start];
+    tmp.len = i->len - start;
+    SECU_PrintAsHex(out, &tmp, m, level);
+    if (unused_bits) {
+	SECU_Indent(out, level + 1);
+	fprintf(out, "(%d least significant bits unused)\n", unused_bits);
+    }
+}
+
 static void
 secu_PrintUniversal(FILE *out, SECItem *i, char *m, int level)
 {
@@ -1019,7 +1074,12 @@ secu_PrintUniversal(FILE *out, SECItem *i, char *m, int level)
           case SEC_ASN1_SEQUENCE:
 	    SECU_PrintSet(out, i, m, level);
 	    break;
-	    
+	  case SEC_ASN1_OCTET_STRING:
+	    secu_PrintOctetString(out, i, m, level);
+	    break;
+	  case SEC_ASN1_BIT_STRING:
+	    secu_PrintBitString(out, i, m, level);
+	    break;
 	  default:
 	    SECU_PrintAsHex(out, i, m, level);
 	    break;
@@ -1029,7 +1089,7 @@ secu_PrintUniversal(FILE *out, SECItem *i, char *m, int level)
 static void
 secu_PrintAny(FILE *out, SECItem *i, char *m, int level)
 {
-    if ( i->len ) {
+    if ( i && i->len && i->data ) {
 	switch (i->data[0] & SEC_ASN1_CLASS_MASK) {
 	case SEC_ASN1_CONTEXT_SPECIFIC:
 	    secu_PrintContextSpecific(out, i, m, level);
@@ -1158,36 +1218,33 @@ secu_PrintSubjectPublicKeyInfo(FILE *out, PRArenaPool *arena,
 		       CERTSubjectPublicKeyInfo *i,  char *msg, int level)
 {
     SECKEYPublicKey *pk;
-    int rv;
 
     SECU_Indent(out, level); fprintf(out, "%s:\n", msg);
     SECU_PrintAlgorithmID(out, &i->algorithm, "Public Key Algorithm", level+1);
 
-    pk = (SECKEYPublicKey*) PORT_ZAlloc(sizeof(SECKEYPublicKey));
-    if (!pk)
-	return PORT_GetError();
+    pk = SECKEY_ExtractPublicKey(i);
+    if (pk) {
+	switch (pk->keyType) {
+	case rsaKey:
+	    secu_PrintRSAPublicKey(out, pk, "RSA Public Key", level +1);
+	    break;
 
-    DER_ConvertBitString(&i->subjectPublicKey);
-    switch(SECOID_FindOIDTag(&i->algorithm.algorithm)) {
-      case SEC_OID_PKCS1_RSA_ENCRYPTION:
-	rv = SEC_ASN1DecodeItem(arena, pk, 
-	                        SEC_ASN1_GET(SECKEY_RSAPublicKeyTemplate),
-				&i->subjectPublicKey);
-	if (rv)
-	    return rv;
-	secu_PrintRSAPublicKey(out, pk, "RSA Public Key", level +1);
-	break;
-      case SEC_OID_ANSIX9_DSA_SIGNATURE:
-	rv = SEC_ASN1DecodeItem(arena, pk, 
-	                        SEC_ASN1_GET(SECKEY_DSAPublicKeyTemplate),
-				&i->subjectPublicKey);
-	if (rv)
-	    return rv;
-	secu_PrintDSAPublicKey(out, pk, "DSA Public Key", level +1);
-	break;
-      default:
-	fprintf(out, "bad SPKI algorithm type\n");
-	return 0;
+	case dsaKey:
+	    secu_PrintDSAPublicKey(out, pk, "DSA Public Key", level +1);
+	    break;
+
+	case dhKey:
+	case fortezzaKey:
+	case keaKey:
+    	    fprintf(out, "unable to format this SPKI algorithm type\n");
+	    break;
+	default:
+	    fprintf(out, "unknown SPKI algorithm type\n");
+	    break;
+	}
+	PORT_FreeArena(pk->arena, PR_FALSE);
+    } else {
+	SECU_PrintError("Error", "Parsing public key");
     }
 
     return 0;
@@ -1459,6 +1516,7 @@ SECU_PrintExtensions(FILE *out, CERTCertExtension **extensions,
 		case SEC_OID_X509_BASIC_CONSTRAINTS:
 		    secu_PrintBasicConstraints(out,tmpitem,"Data",level+1);
 		    break;
+
 		case SEC_OID_X509_SUBJECT_ALT_NAME:
 		case SEC_OID_X509_ISSUER_ALT_NAME:
 	      /*
@@ -1482,12 +1540,13 @@ SECU_PrintExtensions(FILE *out, CERTCertExtension **extensions,
 		case SEC_OID_X509_POLICY_MAPPINGS:
 		case SEC_OID_X509_POLICY_CONSTRAINTS:
 		case SEC_OID_X509_AUTH_KEY_ID:
-            goto defualt;
-		case SEC_OID_X509_EXT_KEY_USAGE:
-            PrintExtKeyUsageExten(out, tmpitem, "", level+1);
-            break;
-		case SEC_OID_X509_AUTH_INFO_ACCESS:
+		    goto defualt;
 
+		case SEC_OID_X509_EXT_KEY_USAGE:
+		    PrintExtKeyUsageExten(out, tmpitem, "", level+1);
+		    break;
+
+		case SEC_OID_X509_AUTH_INFO_ACCESS:
 		case SEC_OID_X509_CRL_NUMBER:
 		case SEC_OID_X509_REASON_CODE:
 
@@ -1514,13 +1573,12 @@ SECU_PrintExtensions(FILE *out, CERTCertExtension **extensions,
 		case SEC_OID_EXT_KEY_USAGE_EMAIL_PROTECT:
 		case SEC_OID_EXT_KEY_USAGE_TIME_STAMP:
 
-	      default:
+	        default:
           defualt:
-		/*SECU_PrintAsHex(out, tmpitem, "Data", level+1); */
 		secu_PrintAny(out, tmpitem, "Data", level+1);
 		break;
 	    }
-		    
+
 	    secu_Newline(out);
 	    extensions++;
 	}
@@ -1605,28 +1663,25 @@ SECU_PrintCertNickname(CERTCertificate *cert, void *data)
     return (SECSuccess);
 }
 
-int
+int  /* sometimes a PRErrorCode, other times a SECStatus.  Sigh. */
 SECU_PrintCertificateRequest(FILE *out, SECItem *der, char *m, int level)
 {
-    PRArenaPool *arena = NULL;
+    PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     CERTCertificateRequest *cr;
-    int rv;
+    int rv = SEC_ERROR_NO_MEMORY;
+
+    if (!arena) 
+	return rv;
 
     /* Decode certificate request */
-    cr = (CERTCertificateRequest*) PORT_ZAlloc(sizeof(CERTCertificateRequest));
+    cr = PORT_ArenaZNew(arena, CERTCertificateRequest);
     if (!cr)
-	return PORT_GetError();
-
-    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-    if (!arena)
-	return SEC_ERROR_NO_MEMORY;
-
-    rv = SEC_ASN1DecodeItem(arena, cr, 
-                            SEC_ASN1_GET(CERT_CertificateRequestTemplate), der);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
-    }
+	goto loser;
+    cr->arena = arena;
+    rv = SEC_QuickDERDecodeItem(arena, cr, 
+                           SEC_ASN1_GET(CERT_CertificateRequestTemplate), der);
+    if (rv) 
+	goto loser;
 
     /* Pretty print it out */
     SECU_Indent(out, level); fprintf(out, "%s:\n", m);
@@ -1634,43 +1689,40 @@ SECU_PrintCertificateRequest(FILE *out, SECItem *der, char *m, int level)
     SECU_PrintName(out, &cr->subject, "Subject", level+1);
     rv = secu_PrintSubjectPublicKeyInfo(out, arena, &cr->subjectPublicKeyInfo,
 			      "Subject Public Key Info", level+1);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
-    }
-    secu_PrintAny(out, cr->attributes[0], "Attributes", level+1);
+    if (rv) 
+	goto loser;
+    if (cr->attributes)
+	secu_PrintAny(out, cr->attributes[0], "Attributes", level+1);
 
+loser:
     PORT_FreeArena(arena, PR_FALSE);
-    return 0;
+    return rv;
 }
 
 int
 SECU_PrintCertificate(FILE *out, SECItem *der, char *m, int level)
 {
-    PRArenaPool *arena = NULL;
+    PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     CERTCertificate *c;
-    int rv;
+    int rv = SEC_ERROR_NO_MEMORY;
     int iv;
     
-    /* Decode certificate */
-    c = (CERTCertificate*) PORT_ZAlloc(sizeof(CERTCertificate));
-    if (!c)
-	return PORT_GetError();
-
-    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     if (!arena)
-	return SEC_ERROR_NO_MEMORY;
+	return rv;
 
+    /* Decode certificate */
+    c = PORT_ArenaZNew(arena, CERTCertificate);
+    if (!c)
+	goto loser;
+    c->arena = arena;
     rv = SEC_ASN1DecodeItem(arena, c, 
                             SEC_ASN1_GET(CERT_CertificateTemplate), der);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
-    }
+    if (rv)
+	goto loser;
 
     /* Pretty print it out */
     SECU_Indent(out, level); fprintf(out, "%s:\n", m);
-    iv = DER_GetInteger(&c->version);
+    iv = c->version.len ? DER_GetInteger(&c->version) : 0;  /* version is optional */
     SECU_Indent(out, level+1); fprintf(out, "%s: %d (0x%x)\n", "Version", iv + 1, iv);
 
     SECU_PrintInteger(out, &c->serialNumber, "Serial Number", level+1);
@@ -1680,72 +1732,62 @@ SECU_PrintCertificate(FILE *out, SECItem *der, char *m, int level)
     SECU_PrintName(out, &c->subject, "Subject", level+1);
     rv = secu_PrintSubjectPublicKeyInfo(out, arena, &c->subjectPublicKeyInfo,
 			      "Subject Public Key Info", level+1);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
-    }
+    if (rv)
+	goto loser;
     SECU_PrintExtensions(out, c->extensions, "Signed Extensions", level+1);
-
     SECU_PrintFingerprints(out, &c->derCert, "Fingerprint", level);
-    
+loser:
     PORT_FreeArena(arena, PR_FALSE);
-    return 0;
+    return rv;
 }
 
 int
 SECU_PrintPublicKey(FILE *out, SECItem *der, char *m, int level)
 {
-    PRArenaPool *arena = NULL;
+    PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     SECKEYPublicKey key;
-    int rv;
+    int rv = SEC_ERROR_NO_MEMORY;
+
+    if (!arena)
+	return rv;
 
     PORT_Memset(&key, 0, sizeof(key));
-    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-    if (!arena)
-	return SEC_ERROR_NO_MEMORY;
-
     rv = SEC_ASN1DecodeItem(arena, &key, 
                             SEC_ASN1_GET(SECKEY_RSAPublicKeyTemplate), der);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
+    if (!rv) {
+	/* Pretty print it out */
+	secu_PrintRSAPublicKey(out, &key, m, level);
     }
 
-    /* Pretty print it out */
-    secu_PrintRSAPublicKey(out, &key, m, level);
-
     PORT_FreeArena(arena, PR_FALSE);
-    return 0;
+    return rv;
 }
 
 #ifdef HAVE_EPV_TEMPLATE
 int
 SECU_PrintPrivateKey(FILE *out, SECItem *der, char *m, int level)
 {
-    PRArenaPool *arena = NULL;
+    PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     SECKEYEncryptedPrivateKeyInfo key;
-    int rv;
+    int rv = SEC_ERROR_NO_MEMORY;
+
+    if (!arena)
+	return rv;
 
     PORT_Memset(&key, 0, sizeof(key));
-    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-    if (!arena)
-	return SEC_ERROR_NO_MEMORY;
-
     rv = SEC_ASN1DecodeItem(arena, &key, 
 		SEC_ASN1_GET(SECKEY_EncryptedPrivateKeyInfoTemplate), der);
-    if (rv) {
-	PORT_FreeArena(arena, PR_TRUE);
-	return rv;
-    }
+    if (rv)
+	goto loser;
 
     /* Pretty print it out */
     SECU_Indent(out, level); fprintf(out, "%s:\n", m);
     SECU_PrintAlgorithmID(out, &key.algorithm, "Encryption Algorithm", 
 			  level+1);
     SECU_PrintAsHex(out, &key.encryptedData, "Encrypted Data", level+1);
-
+loser:
     PORT_FreeArena(arena, PR_TRUE);
-    return 0;
+    return rv;
 }
 #endif
 
@@ -1774,7 +1816,7 @@ SECU_PrintFingerprints(FILE *out, SECItem *derCert, char *m, int level)
     SECU_Indent(out, level);  fprintf(out, "%s (SHA1):\n", m);
     SECU_Indent(out, level+1); fprintf(out, "%s\n", fpStr);
     PORT_Free(fpStr);
-	fprintf(out, "\n");
+    fprintf(out, "\n");
     return 0;
 }
 
@@ -1879,6 +1921,11 @@ secu_PrintSignerInfo(FILE *out, SEC_PKCS7SignerInfo *info, char *m, int level)
     }
 }
 
+/* callers of this function must make sure that the CERTSignedCrl
+   from which they are extracting the CERTCrl has been fully-decoded.
+   Otherwise it will not have the entries even though the CRL may have
+   some */
+
 void
 SECU_PrintCRLInfo(FILE *out, CERTCrl *crl, char *m, int level)
 {
@@ -1915,7 +1962,8 @@ SECU_PrintCRLInfo(FILE *out, CERTCrl *crl, char *m, int level)
 **   Pretty print a PKCS7 signed data type (up to version 1).
 */
 static int
-secu_PrintPKCS7Signed(FILE *out, SEC_PKCS7SignedData *src, char *m, int level)
+secu_PrintPKCS7Signed(FILE *out, SEC_PKCS7SignedData *src,
+		      const char *m, int level)
 {
     SECAlgorithmID *digAlg;		/* digest algorithms */
     SECItem *aCert;			/* certificate */
@@ -1994,7 +2042,7 @@ secu_PrintPKCS7Signed(FILE *out, SEC_PKCS7SignedData *src, char *m, int level)
 */
 static void
 secu_PrintPKCS7Enveloped(FILE *out, SEC_PKCS7EnvelopedData *src,
-			 char *m, int level)
+			 const char *m, int level)
 {
     SEC_PKCS7RecipientInfo *recInfo;   /* pointer for signer information */
     int iv;
@@ -2025,7 +2073,7 @@ secu_PrintPKCS7Enveloped(FILE *out, SEC_PKCS7EnvelopedData *src,
 static int
 secu_PrintPKCS7SignedAndEnveloped(FILE *out,
 				  SEC_PKCS7SignedAndEnvelopedData *src,
-				  char *m, int level)
+				  const char *m, int level)
 {
     SECAlgorithmID *digAlg;  /* pointer for digest algorithms */
     SECItem *aCert;           /* pointer for certificate */
@@ -2110,23 +2158,17 @@ secu_PrintPKCS7SignedAndEnveloped(FILE *out,
 int
 SECU_PrintCrl (FILE *out, SECItem *der, char *m, int level)
 {
-    PRArenaPool *arena = NULL;
+    PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     CERTCrl *c = NULL;
-    int rv;
+    int rv = SEC_ERROR_NO_MEMORY;
 
+    if (!arena)
+    	return rv;
     do {
 	/* Decode CRL */
-	c = (CERTCrl*) PORT_ZAlloc(sizeof(CERTCrl));
-	if (!c) {
-	    rv = PORT_GetError();
+	c = PORT_ArenaZNew(arena, CERTCrl);
+	if (!c)
 	    break;
-	}
-
-	arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-	if (!arena) {
-	    rv = SEC_ERROR_NO_MEMORY;
-	    break;
-	}
 
 	rv = SEC_ASN1DecodeItem(arena, c, SEC_ASN1_GET(CERT_CrlTemplate), der);
 	if (rv != SECSuccess)
@@ -2134,8 +2176,7 @@ SECU_PrintCrl (FILE *out, SECItem *der, char *m, int level)
 	SECU_PrintCRLInfo (out, c, m, level);
     } while (0);
     PORT_FreeArena (arena, PR_FALSE);
-    PORT_Free (c);
-    return (rv);
+    return rv;
 }
 
 
@@ -2145,7 +2186,7 @@ SECU_PrintCrl (FILE *out, SECItem *der, char *m, int level)
 */
 static void
 secu_PrintPKCS7Encrypted(FILE *out, SEC_PKCS7EncryptedData *src,
-			 char *m, int level)
+			 const char *m, int level)
 {
     SECU_Indent(out, level); fprintf(out, "%s:\n", m);
     SECU_PrintInteger(out, &(src->version), "Version", level + 1);
@@ -2160,7 +2201,7 @@ secu_PrintPKCS7Encrypted(FILE *out, SEC_PKCS7EncryptedData *src,
 */
 static void
 secu_PrintPKCS7Digested(FILE *out, SEC_PKCS7DigestedData *src,
-			char *m, int level)
+			const char *m, int level)
 {
     SECU_Indent(out, level); fprintf(out, "%s:\n", m);
     SECU_PrintInteger(out, &(src->version), "Version", level + 1);
@@ -2181,7 +2222,7 @@ static int
 secu_PrintPKCS7ContentInfo(FILE *out, SEC_PKCS7ContentInfo *src,
 			   char *m, int level)
 {
-    char *desc;
+    const char *desc;
     SECOidTag kind;
     int rv;
 
@@ -2313,40 +2354,35 @@ SECU_PrintTrustFlags(FILE *out, CERTCertTrust *trust, char *m, int level)
 int SECU_PrintSignedData(FILE *out, SECItem *der, char *m,
 			   int level, SECU_PPFunc inner)
 {
-    PRArenaPool *arena = NULL;
+    PRArenaPool *arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     CERTSignedData *sd;
-    int rv;
+    int rv = SEC_ERROR_NO_MEMORY;
+
+    if (!arena)
+	return rv;
 
     /* Strip off the signature */
-    sd = (CERTSignedData*) PORT_ZAlloc(sizeof(CERTSignedData));
+    sd = PORT_ArenaZNew(arena, CERTSignedData);
     if (!sd)
-	return PORT_GetError();
-
-    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-    if (!arena)
-	return SEC_ERROR_NO_MEMORY;
+	goto loser;
 
     rv = SEC_ASN1DecodeItem(arena, sd, SEC_ASN1_GET(CERT_SignedDataTemplate), 
                             der);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
-    }
+    if (rv)
+	goto loser;
 
     SECU_Indent(out, level); fprintf(out, "%s:\n", m);
     rv = (*inner)(out, &sd->data, "Data", level+1);
-    if (rv) {
-	PORT_FreeArena(arena, PR_FALSE);
-	return rv;
-    }
+    if (rv) 
+	goto loser;
 
     SECU_PrintAlgorithmID(out, &sd->signatureAlgorithm, "Signature Algorithm",
 			  level+1);
     DER_ConvertBitString(&sd->signature);
     SECU_PrintAsHex(out, &sd->signature, "Signature", level+1);
-
+loser:
     PORT_FreeArena(arena, PR_FALSE);
-    return 0;
+    return rv;
 
 }
 
@@ -2574,7 +2610,6 @@ void
 SECU_PrintPRandOSError(char *progName) 
 {
     char buffer[513];
-    PRErrorCode err    = PR_GetError();
     PRInt32     errLen = PR_GetErrorTextLength();
     if (errLen > 0 && errLen < sizeof buffer) {
         PR_GetErrorText(buffer);
