@@ -38,143 +38,37 @@
 #include "nssilock.h"
 #include "secmodi.h"
 #include "pk11func.h"
+#include "pki3hack.h"
+#include "secerr.h"
 
 /* these are for displaying error messages */
 
 static  SECMODModuleList *modules = NULL;
+static  SECMODModuleList *modulesDB = NULL;
+static  SECMODModuleList *modulesUnload = NULL;
 static  SECMODModule *internalModule = NULL;
+static  SECMODModule *defaultDBModule = NULL;
+static  SECMODModule *pendingModule = NULL;
 static SECMODListLock *moduleLock = NULL;
 
-extern SECStatus
-PK11_UpdateSlotAttribute(PK11SlotInfo *slot, PK11DefaultArrayEntry *entry,
-                        PRBool add);
-                        
-
-extern int XP_SEC_MODULE_NO_LIB;
+int secmod_PrivateModuleCount = 0;
 
 extern PK11DefaultArrayEntry PK11_DefaultArray[];
 extern int num_pk11_default_mechanisms;
 
-static PRBool secmod_ModuleHasRoots(SECMODModule *module)
-{
-    int i;
-
-    for (i=0; i < module->slotInfoCount; i++) {
-	if (module->slotInfo[i].hasRootCerts) {
-	    return PR_TRUE;
-	}
-    }
-    return PR_FALSE;
-}
-
-
-/*
- * The following code is an attempt to automagically find the external root
- * module. NOTE: This code should be checked out on the MAC! There must be
- * some cross platform support out there to help out with this?
- */
-
-static char *dllnames[]= {
-	"roots.dll", "libroots.so","libroots.sl","Root Certs",
-	"roots.dll", "libroots.so","libroots.sl","Root Certs",
-	"nssckbi.dll","libnssckbi.so","libnssckbi.sl","NSS Builtin Root Certs",
-	"mozckbi.dll","libmozckbi.so","libmozckbi.sl","Mozilla Builtin Root Certs",
-	"netckbi.dll","libnetckbi.so","libnetckbi.sl","Netscape Builtin Root Certs",
-	0 };
-
-#define MAXDLLNAME 40
-
-/* Should we have platform ifdefs here??? */
-#define FILE_SEP '/'
-
-static void
-secmod_FindExternalRoot(char *dbname)
-{
-	char *path, *cp, **cur_name;
-	int len = PORT_Strlen(dbname);
-	int path_len;
-
-	
-	path = PORT_Alloc(len+MAXDLLNAME);
-	if (path == NULL) return;
-
-	/* back up to the top of the directory */
-	for (cp = &dbname[len]; cp != dbname && (*cp != FILE_SEP); cp--) ;
-	path_len = cp-dbname;
-	PORT_Memcpy(path,dbname,path_len);
-	path[path_len++] = FILE_SEP;
-
-	/* now walk our tree of dll names looking for the file of interest. */
-	for (cur_name= dllnames; *cur_name != 0; cur_name++) {
-	    PORT_Memcpy(&path[path_len],*cur_name,PORT_Strlen(*cur_name)+1);
-	    if (SECMOD_AddNewModule("Root Certs",path, 0, 0) == SECSuccess) {
-		break;
-	    }
-	}
-	PORT_Free(path);
-	return;
-}
-
-void SECMOD_init(char *dbname) {
-    SECMODModuleList *thisModule;
-    int found=0;
-    int rootFound=0;
-    SECStatus rv = SECFailure;
-
 
+void
+SECMOD_Init() {
     /* don't initialize twice */
-    if (modules) return;
-
-    PK11_InitSlotLists();
-
-    SECMOD_InitDB(dbname);
-
-    /*
-     * read in the current modules from the database
-     */
-    modules = SECMOD_ReadPermDB();
-
-    /* make sure that the internal module is loaded */
-    for (thisModule = modules; thisModule ; thisModule = thisModule->next) {
-	if (thisModule->module->internal) {
-	    found++;
-	    internalModule = SECMOD_ReferenceModule(thisModule->module);
-	}
-	if (secmod_ModuleHasRoots(thisModule->module)) {
-	    rootFound++;
-	}
-    }
-
-    if (!found) {
-	thisModule = modules;
-	modules = SECMOD_NewModuleListElement();
-	modules->module = SECMOD_NewInternal();
-	PORT_Assert(modules->module != NULL);
-	modules->next = thisModule;
-	SECMOD_AddPermDB(modules->module);
-	internalModule = SECMOD_ReferenceModule(modules->module);
-    }
-
-    /* load it first... we need it to verify the external modules
-     * which we are loading.... */
-    rv = SECMOD_LoadModule(internalModule);
-    if( rv != SECSuccess )
-        internalModule = NULL;
-
-    if (! rootFound ) {
-	secmod_FindExternalRoot(dbname);
-    }
-    /* Load each new module */
-    for (thisModule = modules; thisModule ; thisModule = thisModule->next) {
-        if( !( thisModule->module->internal ) )
-	    SECMOD_LoadModule(thisModule->module);
-    }
+    if (moduleLock) return;
 
     moduleLock = SECMOD_NewListLock();
+    PK11_InitSlotLists();
 }
 
 
-void SECMOD_Shutdown() {
+SECStatus
+SECMOD_Shutdown() {
     /* destroy the lock */
     if (moduleLock) {
 	SECMOD_DestroyListLock(moduleLock);
@@ -185,14 +79,44 @@ void SECMOD_Shutdown() {
 	SECMOD_DestroyModule(internalModule);
 	internalModule = NULL;
     }
+
+    /* free the default database module */
+    if (defaultDBModule) {
+	SECMOD_DestroyModule(defaultDBModule);
+	defaultDBModule = NULL;
+    }
+	
     /* destroy the list */
     if (modules) {
 	SECMOD_DestroyModuleList(modules);
 	modules = NULL;
     }
+   
+    if (modulesDB) {
+	SECMOD_DestroyModuleList(modulesDB);
+	modulesDB = NULL;
+    }
+
+    if (modulesUnload) {
+	SECMOD_DestroyModuleList(modulesUnload);
+	modulesUnload = NULL;
+    }
 
     /* make all the slots and the lists go away */
     PK11_DestroySlotLists();
+
+    nss_DumpModuleLog();
+
+#ifdef DEBUG
+    if (PR_GetEnv("NSS_STRICT_SHUTDOWN")) {
+	PORT_Assert(secmod_PrivateModuleCount == 0);
+    }
+#endif
+    if (secmod_PrivateModuleCount) {
+    	PORT_SetError(SEC_ERROR_BUSY);
+	return SECFailure;
+    }
+    return SECSuccess;
 }
 
 
@@ -204,14 +128,54 @@ SECMOD_GetInternalModule(void) {
    return internalModule;
 }
 
-/* called from  security/cmd/swfort/instinit, which doesn't need a full 
- * security LIBRARY (it used the swfortezza code, but it does have to verify
- * cert chains against it's own list of certs. We need to initialize the 
- * security code without any database.
- */
-void
-SECMOD_SetInternalModule( SECMODModule *mod) {
-   internalModule = SECMOD_ReferenceModule(mod);
+
+SECStatus
+secmod_AddModuleToList(SECMODModuleList **moduleList,SECMODModule *newModule) {
+    SECMODModuleList *mlp, *newListElement, *last = NULL;
+
+    newListElement = SECMOD_NewModuleListElement();
+    if (newListElement == NULL) {
+	return SECFailure;
+    }
+
+    newListElement->module = SECMOD_ReferenceModule(newModule);
+
+    SECMOD_GetWriteLock(moduleLock);
+    /* Added it to the end (This is very inefficient, but Adding a module
+     * on the fly should happen maybe 2-3 times through the life this program
+     * on a given computer, and this list should be *SHORT*. */
+    for(mlp = *moduleList; mlp != NULL; mlp = mlp->next) {
+	last = mlp;
+    }
+
+    if (last == NULL) {
+	*moduleList = newListElement;
+    } else {
+	SECMOD_AddList(last,newListElement,NULL);
+    }
+    SECMOD_ReleaseWriteLock(moduleLock);
+    return SECSuccess;
+}
+
+SECStatus
+SECMOD_AddModuleToList(SECMODModule *newModule) {
+    if (newModule->internal && !internalModule) {
+	internalModule = SECMOD_ReferenceModule(newModule);
+    }
+    return secmod_AddModuleToList(&modules,newModule);
+}
+
+SECStatus
+SECMOD_AddModuleToDBOnlyList(SECMODModule *newModule) {
+    if (defaultDBModule == NULL) {
+	defaultDBModule = SECMOD_ReferenceModule(newModule);
+    }
+    return secmod_AddModuleToList(&modulesDB,newModule);
+}
+
+SECStatus
+SECMOD_AddModuleToUnloadList(SECMODModule *newModule) {
+    return secmod_AddModuleToList(&modulesUnload,newModule);
 }
 
 /*
@@ -288,10 +252,11 @@ PK11SlotInfo *SECMOD_LookupSlot(SECMODModuleID moduleID,CK_SLOT_ID slotID) {
 
 
 /*
- * find a module by name and delete it of the module list
+ * find a module by name or module pointer and delete it off the module list.
+ * optionally remove it from secmod.db.
  */
 SECStatus
-SECMOD_DeleteModule(char *name, int *type) {
+SECMOD_DeleteModuleEx(char *name, SECMODModule *mod, int *type, PRBool permdb) {
     SECMODModuleList *mlp;
     SECMODModuleList **mlpp;
     SECStatus rv = SECFailure;
@@ -302,12 +267,13 @@ SECMOD_DeleteModule(char *name, int *type) {
     SECMOD_GetWriteLock(moduleLock);
     for(mlpp = &modules,mlp = modules; 
 				mlp != NULL; mlpp = &mlp->next, mlp = *mlpp) {
-	if (PORT_Strcmp(name,mlp->module->commonName) == 0) {
+	if ((name && (PORT_Strcmp(name,mlp->module->commonName) == 0)) ||
+							mod == mlp->module) {
 	    /* don't delete the internal module */
 	    if (!mlp->module->internal) {
 		SECMOD_RemoveList(mlpp,mlp);
 		/* delete it after we release the lock */
-		rv = SECSuccess;
+		rv = STAN_RemoveModuleFromDefaultTrustDomain(mlp->module);
 	    } else if (mlp->module->isFIPS) {
 		*type = SECMOD_FIPS;
 	    } else {
@@ -320,14 +286,24 @@ SECMOD_DeleteModule(char *name, int *type) {
 
 
     if (rv == SECSuccess) {
- 	SECMOD_DeletePermDB(mlp->module);
+	if (permdb) {
+ 	    SECMOD_DeletePermDB(mlp->module);
+	}
 	SECMOD_DestroyModuleListElement(mlp);
     }
     return rv;
 }
 
 /*
- * find a module by name and delete it of the module list
+ * find a module by name and delete it off the module list
+ */
+SECStatus
+SECMOD_DeleteModule(char *name, int *type) {
+    return SECMOD_DeleteModuleEx(name, NULL, type, PR_TRUE);
+}
+
+/*
+ * find a module by name and delete it off the module list
  */
 SECStatus
 SECMOD_DeleteInternalModule(char *name) {
@@ -335,14 +311,19 @@ SECMOD_DeleteInternalModule(char *name) {
     SECMODModuleList **mlpp;
     SECStatus rv = SECFailure;
 
+    if (pendingModule) {
+	PORT_SetError(SEC_ERROR_MODULE_STUCK);
+	return rv;
+    }
+
     SECMOD_GetWriteLock(moduleLock);
     for(mlpp = &modules,mlp = modules; 
 				mlp != NULL; mlpp = &mlp->next, mlp = *mlpp) {
 	if (PORT_Strcmp(name,mlp->module->commonName) == 0) {
 	    /* don't delete the internal module */
 	    if (mlp->module->internal) {
-		rv = SECSuccess;
 		SECMOD_RemoveList(mlpp,mlp);
+		rv = STAN_RemoveModuleFromDefaultTrustDomain(mlp->module);
 	    } 
 	    break;
 	}
@@ -353,12 +334,14 @@ SECMOD_DeleteInternalModule(char *name) {
 	SECMODModule *newModule,*oldModule;
 
 	if (mlp->module->isFIPS) {
-	    newModule = SECMOD_NewInternal();
+    	    newModule = SECMOD_CreateModule(NULL, SECMOD_INT_NAME,
+				NULL, SECMOD_INT_FLAGS);
 	} else {
-	    newModule = SECMOD_GetFIPSInternal();
+    	    newModule = SECMOD_CreateModule(NULL, SECMOD_FIPS_NAME,
+				NULL, SECMOD_FIPS_FLAGS);
 	}
 	if (newModule == NULL) {
-	    SECMODModuleList *last,*mlp2;
+	    SECMODModuleList *last = NULL,*mlp2;
 	   /* we're in pretty deep trouble if this happens...Security
 	    * not going to work well... try to put the old module back on
 	    * the list */
@@ -375,12 +358,15 @@ SECMOD_DeleteInternalModule(char *name) {
 	   SECMOD_ReleaseWriteLock(moduleLock);
 	   return SECFailure; 
 	}
-	oldModule = internalModule;
-	internalModule = SECMOD_ReferenceModule(newModule);
-	SECMOD_AddModule(internalModule);
+	newModule->libraryParams = 
+	     PORT_ArenaStrdup(newModule->arena,mlp->module->libraryParams);
+	pendingModule = oldModule = internalModule;
+	internalModule = NULL;
 	SECMOD_DestroyModule(oldModule);
  	SECMOD_DeletePermDB(mlp->module);
 	SECMOD_DestroyModuleListElement(mlp);
+	internalModule = newModule; /* adopt the module */
+	SECMOD_AddModule(internalModule);
     }
     return rv;
 }
@@ -388,47 +374,34 @@ SECMOD_DeleteInternalModule(char *name) {
 SECStatus
 SECMOD_AddModule(SECMODModule *newModule) {
     SECStatus rv;
-    SECMODModuleList *mlp, *newListElement, *last = NULL;
+    SECMODModule *oldModule;
 
     /* Test if a module w/ the same name already exists */
     /* and return SECWouldBlock if so. */
     /* We should probably add a new return value such as */
     /* SECDublicateModule, but to minimize ripples, I'll */
     /* give SECWouldBlock a new meaning */
-    if (SECMOD_FindModule(newModule->commonName)) {
+    if ((oldModule = SECMOD_FindModule(newModule->commonName)) != NULL) {
+	SECMOD_DestroyModule(oldModule);
         return SECWouldBlock;
         /* module already exists. */
     }
 
-    rv = SECMOD_LoadModule(newModule);
+    rv = SECMOD_LoadPKCS11Module(newModule);
     if (rv != SECSuccess) {
 	return rv;
     }
 
-    newListElement = SECMOD_NewModuleListElement();
-    if (newListElement == NULL) {
-	return SECFailure;
+    if (newModule->parent == NULL) {
+	newModule->parent = SECMOD_ReferenceModule(defaultDBModule);
     }
 
     SECMOD_AddPermDB(newModule);
+    SECMOD_AddModuleToList(newModule);
 
-    newListElement->module = newModule;
+    rv = STAN_AddModuleToDefaultTrustDomain(newModule);
 
-    SECMOD_GetWriteLock(moduleLock);
-    /* Added it to the end (This is very inefficient, but Adding a module
-     * on the fly should happen maybe 2-3 times through the life this program
-     * on a given computer, and this list should be *SHORT*. */
-    for(mlp = modules; mlp != NULL; mlp = mlp->next) {
-	last = mlp;
-    }
-
-    if (last == NULL) {
-	modules = newListElement;
-    } else {
-	SECMOD_AddList(last,newListElement,NULL);
-    }
-    SECMOD_ReleaseWriteLock(moduleLock);
-    return SECSuccess;
+    return rv;
 }
 
 PK11SlotInfo *SECMOD_FindSlot(SECMODModule *module,char *name) {
@@ -451,11 +424,15 @@ PK11SlotInfo *SECMOD_FindSlot(SECMODModule *module,char *name) {
 }
 
 SECStatus
-PK11_GetModInfo(SECMODModule *mod,CK_INFO *info) {
+PK11_GetModInfo(SECMODModule *mod,CK_INFO *info)
+{
     CK_RV crv;
 
     if (mod->functionList == NULL) return SECFailure;
     crv = PK11_GETTAB(mod)->C_GetInfo(info);
+    if (crv != CKR_OK) {
+	PORT_SetError(PK11_MapError(crv));
+    }	
     return (crv == CKR_OK) ? SECSuccess : SECFailure;
 }
 
@@ -478,36 +455,28 @@ PK11_IsFIPS(void)
 /* combines NewModule() & AddModule */
 /* give a string for the module name & the full-path for the dll, */
 /* installs the PKCS11 module & update registry */
-SECStatus SECMOD_AddNewModule(char* moduleName, char* dllPath,
+SECStatus SECMOD_AddNewModuleEx(char* moduleName, char* dllPath,
                               unsigned long defaultMechanismFlags,
-                              unsigned long cipherEnableFlags) {
+                              unsigned long cipherEnableFlags,
+                              char* modparms,
+                              char* nssparms) {
     SECMODModule *module;
-    SECStatus result;
+    SECStatus result = SECFailure;
     int s,i;
     PK11SlotInfo* slot;
 
-    module = SECMOD_NewModule();
-    
-    if (moduleName) {	
-        module->commonName=PORT_ArenaStrdup(module->arena,moduleName);
-    } else {
-        module->commonName=NULL;
-    }
+    PR_SetErrorText(0, NULL);
 
-    if (dllPath) {
-        module->dllName=PORT_ArenaStrdup(module->arena,dllPath);
-    } else {
-        module->dllName=NULL;
+    module = SECMOD_CreateModule(dllPath,moduleName, modparms, nssparms);
+
+    if (module == NULL) {
+	return result;
     }
 
     if (module->dllName != NULL) {
         if (module->dllName[0] != 0) {
-           SECStatus rv = SECMOD_AddModule(module);
-            if (rv != SECSuccess) {
-                /* SECFailure: failed to add module, corrupt or missing module etc. */
-                /* SECBlock: a module with the same name already exists */
-                return rv;
-            } else { /* successfully added module */
+            result = SECMOD_AddModule(module);
+            if (result == SECSuccess) {
                 /* turn on SSL cipher enable flags */
                 module->ssl[0] = cipherEnableFlags;
 
@@ -531,20 +500,33 @@ SECStatus SECMOD_AddNewModule(char* moduleName, char* dllPath,
                 } /* for each slot of this module */
 
                 /* delete and re-add module in order to save changes to the module */
-                result = SECMOD_DeletePermDB(module);
-                
-                if (result == SECSuccess) {          
-                    result = SECMOD_AddPermDB(module);
-                    if (result == SECSuccess) {
-                        return SECSuccess;
-                    }                    
-                }
-                
+		result = SECMOD_UpdateModule(module);
             }
         }
     }
     SECMOD_DestroyModule(module);
-    return SECFailure;
+    return result;
+}
+
+SECStatus SECMOD_AddNewModule(char* moduleName, char* dllPath,
+                              unsigned long defaultMechanismFlags,
+                              unsigned long cipherEnableFlags)
+{
+    return SECMOD_AddNewModuleEx(moduleName, dllPath, defaultMechanismFlags,
+                  cipherEnableFlags, 
+                  NULL, NULL); /* don't pass module or nss params */
+}
+
+SECStatus SECMOD_UpdateModule(SECMODModule *module)
+{
+    SECStatus result;
+
+    result = SECMOD_DeletePermDB(module);
+                
+    if (result == SECSuccess) {          
+	result = SECMOD_AddPermDB(module);
+    }
+    return result;
 }
 
 /* Public & Internal(Security Library)  representation of
@@ -592,7 +574,6 @@ SECMOD_IsModulePresent( unsigned long int pubCipherEnableFlags )
 {
     PRBool result = PR_FALSE;
     SECMODModuleList *mods = SECMOD_GetDefaultModuleList();
-    SECMODListLock *modsLock = SECMOD_GetDefaultModuleListLock();
     SECMOD_GetReadLock(moduleLock);
 
 
@@ -605,3 +586,135 @@ SECMOD_IsModulePresent( unsigned long int pubCipherEnableFlags )
     SECMOD_ReleaseReadLock(moduleLock);
     return result;
 }
+
+/* create a new ModuleListElement */
+SECMODModuleList *SECMOD_NewModuleListElement(void) {
+    SECMODModuleList *newModList;
+
+    newModList= (SECMODModuleList *) PORT_Alloc(sizeof(SECMODModuleList));
+    if (newModList) {
+	newModList->next = NULL;
+	newModList->module = NULL;
+    }
+    return newModList;
+}
+/*
+ * make a new reference to a module so It doesn't go away on us
+ */
+SECMODModule *
+SECMOD_ReferenceModule(SECMODModule *module) {
+    PK11_USE_THREADS(PZ_Lock((PZLock *)module->refLock);)
+    PORT_Assert(module->refCount > 0);
+
+    module->refCount++;
+    PK11_USE_THREADS(PZ_Unlock((PZLock*)module->refLock);)
+    return module;
+}
+
+
+/* destroy an existing module */
+void
+SECMOD_DestroyModule(SECMODModule *module) {
+    PRBool willfree = PR_FALSE;
+    int slotCount;
+    int i;
+
+    PK11_USE_THREADS(PZ_Lock((PZLock *)module->refLock);)
+    if (module->refCount-- == 1) {
+	willfree = PR_TRUE;
+    }
+    PORT_Assert(willfree || (module->refCount > 0));
+    PK11_USE_THREADS(PZ_Unlock((PZLock *)module->refLock);)
+
+    if (!willfree) {
+	return;
+    }
+   
+    if (module->parent != NULL) {
+	SECMODModule *parent = module->parent;
+	/* paranoia, don't loop forever if the modules are looped */
+	module->parent = NULL;
+	SECMOD_DestroyModule(parent);
+    }
+
+    /* slots can't really disappear until our module starts freeing them,
+     * so this check is safe */
+    slotCount = module->slotCount;
+    if (slotCount == 0) {
+	SECMOD_SlotDestroyModule(module,PR_FALSE);
+	return;
+    }
+
+    /* now free all out slots, when they are done, they will cause the
+     * module to disappear altogether */
+    for (i=0 ; i < slotCount; i++) {
+	if (!module->slots[i]->disabled) {
+		PK11_ClearSlotList(module->slots[i]);
+	}
+	PK11_FreeSlot(module->slots[i]);
+    }
+    /* WARNING: once the last slot has been freed is it possible (even likely)
+     * that module is no more... touching it now is a good way to go south */
+}
+
+
+/* we can only get here if we've destroyed the module, or some one has
+ * erroneously freed a slot that wasn't referenced. */
+void
+SECMOD_SlotDestroyModule(SECMODModule *module, PRBool fromSlot) {
+    PRBool willfree = PR_FALSE;
+    if (fromSlot) {
+        PORT_Assert(module->refCount == 0);
+	PK11_USE_THREADS(PZ_Lock((PZLock *)module->refLock);)
+	if (module->slotCount-- == 1) {
+	    willfree = PR_TRUE;
+	}
+	PORT_Assert(willfree || (module->slotCount > 0));
+	PK11_USE_THREADS(PZ_Unlock((PZLock *)module->refLock);)
+        if (!willfree) return;
+    }
+
+    if (module == pendingModule) {
+	pendingModule = NULL;
+    }
+
+    if (module->loaded) {
+	SECMOD_UnloadModule(module);
+    }
+    PK11_USE_THREADS(PZ_DestroyLock((PZLock *)module->refLock);)
+    PORT_FreeArena(module->arena,PR_FALSE);
+    secmod_PrivateModuleCount--;
+}
+
+/* destroy a list element
+ * this destroys a single element, and returns the next element
+ * on the chain. It makes it easy to implement for loops to delete
+ * the chain. It also make deleting a single element easy */
+SECMODModuleList *
+SECMOD_DestroyModuleListElement(SECMODModuleList *element) {
+    SECMODModuleList *next = element->next;
+
+    if (element->module) {
+	SECMOD_DestroyModule(element->module);
+	element->module = NULL;
+    }
+    PORT_Free(element);
+    return next;
+}
+
+
+/*
+ * Destroy an entire module list
+ */
+void
+SECMOD_DestroyModuleList(SECMODModuleList *list) {
+    SECMODModuleList *lp;
+
+    for ( lp = list; lp != NULL; lp = SECMOD_DestroyModuleListElement(lp)) ;
+}
+
+PRBool
+SECMOD_CanDeleteInternalModule(void)
+{
+    return (PRBool) (pendingModule == NULL);
+}
